/*******************************************************************************
* File Name: Comp_ULD.c  
* Version 1.80
*
*  Description:
*    This file provides the source code to the API for the Comparator component.
*
*  Note:
*     
*******************************************************************************
* Copyright 2008-2011, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "Comp_ULD.h"

uint8 Comp_ULD_initVar = 0u;
static Comp_ULD_backupStruct  Comp_ULD_backup;

/* variable to decide whether or not to restore the control register in 
   Enable() API for PSoC5 ES1 only */
#if (CY_PSOC5_ES1)
    uint8 Comp_ULD_restoreReg = 0u;
#endif /* CY_PSOC5_ES1 */


/*******************************************************************************
* Function Name: Comp_ULD_Init
********************************************************************************
*
* Summary:
*  Initialize to the schematic state
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Comp_ULD_Init(void) 
{
    /* Set default speed/power */
    Comp_ULD_SetSpeed(Comp_ULD_DEFAULT_SPEED);

    /* Set default Hysteresis */
    if ( Comp_ULD_DEFAULT_HYSTERESIS == 0u )
    {
        Comp_ULD_CR |= Comp_ULD_HYST_OFF;
    }
    else
    {
        Comp_ULD_CR &= ~Comp_ULD_HYST_OFF; 
    }
    
    /* Power down override feature is not supported for PSoC5. */
    #if (CY_PSOC3)
        /* Set default Power Down Override */
        if ( Comp_ULD_DEFAULT_PWRDWN_OVRD == 0u )
        {
            Comp_ULD_CR &= ~Comp_ULD_PWRDWN_OVRD;
        }
        else 
        {
            Comp_ULD_CR |= Comp_ULD_PWRDWN_OVRD;
        }
    #endif /* CY_PSOC3 */
    
    /* Set mux always on logic */
    Comp_ULD_CR |= Comp_ULD_MX_AO;

    /* Set default sync */
    Comp_ULD_CLK &= ~Comp_ULD_SYNCCLK_MASK;
    if ( Comp_ULD_DEFAULT_BYPASS_SYNC == 0u )
    {
        Comp_ULD_CLK |= Comp_ULD_SYNC_CLK_EN;
    }
    else
    {
        Comp_ULD_CLK |= Comp_ULD_BYPASS_SYNC;
    }
}


/*******************************************************************************
* Function Name: Comp_ULD_Enable
********************************************************************************
*
* Summary:
*  Enable the Comparator
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Comp_ULD_Enable(void) 
{
    Comp_ULD_PWRMGR |= Comp_ULD_ACT_PWR_EN;
    Comp_ULD_STBY_PWRMGR |= Comp_ULD_STBY_PWR_EN;
     
     /* This is to restore the value of register CR which is saved 
    in prior to the modification in stop() API */
    #if (CY_PSOC5_ES1)
    if(Comp_ULD_restoreReg == 1u)
    {
        Comp_ULD_CR = Comp_ULD_backup.compCRReg;

        /* Clear the flag */
        Comp_ULD_restoreReg = 0u;
    }
    #endif /* CY_PSOC5_ES1 */
}


/*******************************************************************************
* Function Name: Comp_ULD_Start
********************************************************************************
*
* Summary:
*  The start function initializes the Analog Comparator with the default values.
*
* Parameters:
*  void
*
* Return:  
*  void 
*
* Global variables:
*  Comp_ULD_initVar: Is modified when this function is called for the 
*    first time. Is used to ensure that initialization happens only once.
*  
* Reentrant:
*  No
*
*******************************************************************************/
void Comp_ULD_Start(void) 
{

    if ( Comp_ULD_initVar == 0u )
    {
        Comp_ULD_Init();
        
        Comp_ULD_initVar = 1u;
    }    

    /* Enable power to comparator */
    Comp_ULD_Enable();    
}


/*******************************************************************************
* Function Name: Comp_ULD_Stop
********************************************************************************
*
* Summary:
*  Powers down amplifier to lowest power state.
*
* Parameters:  
*  void
*
* Return: 
*  void 
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Comp_ULD_Stop(void) 
{
    /* Disable power to comparator */
    Comp_ULD_PWRMGR &= ~Comp_ULD_ACT_PWR_EN;
    Comp_ULD_STBY_PWRMGR &= ~Comp_ULD_STBY_PWR_EN;    

    #if (CY_PSOC5_ES1)
        /* Enable the variable */
        Comp_ULD_restoreReg = 1u;

        /* Save the control register before clearing it */
        Comp_ULD_backup.compCRReg = Comp_ULD_CR;
        Comp_ULD_CR = Comp_ULD_COMP_REG_CLR;
    #endif /* CY_PSOC5_ES1 */
}


/*******************************************************************************
* Function Name: Comp_ULD_SetSpeed
********************************************************************************
*
* Summary:
*  This function sets the speed of the Analog Comparator.  The faster the speed
*  the more power that is used.
*
* Parameters:  
*  speed:   (uint8) Sets operation mode of Comparator
*
* Return:  
*  void 
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Comp_ULD_SetSpeed(uint8 speed) 
{
    /* Clear and Set power level */    
    Comp_ULD_CR = (Comp_ULD_CR & ~Comp_ULD_PWR_MODE_MASK) |
                           (speed & Comp_ULD_PWR_MODE_MASK);

    /* Set trim value for high speed comparator */
    if(speed == Comp_ULD_HIGHSPEED)
    {
        /* PSoC3 ES2 or early, PSoC5 ES1 or early */
        #if (CY_PSOC3_ES2 || CY_PSOC5_ES1)
        Comp_ULD_TR = Comp_ULD_HS_TRIM_TR0;
        #endif /* CY_PSOC3_ES2 || CY_PSOC5_ES1 */
        
        /* PSoC3 ES3 or later, PSoC5 ES2 or later */
        #if (CY_PSOC3_ES3 || CY_PSOC5_ES2) 
        Comp_ULD_TR0 = Comp_ULD_HS_TRIM_TR0;
        Comp_ULD_TR1 = Comp_ULD_HS_TRIM_TR1;
        #endif /* CY_PSOC3_ES2 || CY_PSOC5_ES1 */
    }
    else
    {
    /* PSoC3 ES2 or early, PSoC5 ES1 or early */
        #if (CY_PSOC3_ES2 || CY_PSOC5_ES1)
        Comp_ULD_TR = Comp_ULD_LS_TRIM_TR0;
        #endif /* CY_PSOC3_ES2 || CY_PSOC5_ES1 */
        
        /* PSoC3 ES3 or later, PSoC5 ES2 or later */
        #if (CY_PSOC3_ES3 || CY_PSOC5_ES2) 
        Comp_ULD_TR0 = Comp_ULD_LS_TRIM_TR0;
        Comp_ULD_TR1 = Comp_ULD_LS_TRIM_TR1;
        #endif /* CY_PSOC3_ES2 || CY_PSOC5_ES1 */
    }

}


/*******************************************************************************
* Function Name: Comp_ULD_GetCompare
********************************************************************************
*
* Summary:
*  This function returns the comparator output value.
*
* Parameters:  
*   None
*
* Return:  
*  (uint8)  0  if Pos_Input less than Neg_input
*           1  if Pos_Input greater than Neg_input. 
*
* Reentrant:
*  Yes
*
*******************************************************************************/
uint8 Comp_ULD_GetCompare(void) 
{
    return( Comp_ULD_WRK & Comp_ULD_CMP_OUT_MASK);
}


/*******************************************************************************
* Function Name: Comp_ULD_trimAdjust
********************************************************************************
*
* Summary:
*  This function adjusts the value in the low nibble/high nibble of the Analog 
*  Comparator trim register
*
* Parameters:  
*  nibble:
*      0 -- adjusts the value in the low nibble
*      1 -- adjusts the value in the high nibble
*
* Return:
*  None
*
* Theory: 
*  Function assumes comparator block is setup for trim adjust.
*  Intended to be called from Comp_ZeroCal()
* 
* Side Effects:
*  Routine uses a course 1ms delay following each trim adjustment to allow 
*  the comparator output to respond.
*
* Reentrant:
*  No
*
*******************************************************************************/
void Comp_ULD_trimAdjust(uint8 nibble)
{
    uint8 trimCnt;
    uint8 cmpState;   

    /* get current state of comparator output */
    cmpState = Comp_ULD_WRK & Comp_ULD_CMP_OUT_MASK;
    
    if (nibble == 0u)
    {    
        /* if comparator output is high, negative offset adjust is required */
        if ( cmpState != 0u )
        {
            /* PSoC3 ES2 or early, PSoC5 ES1 or early */
            #if (CY_PSOC3_ES2 || CY_PSOC5_ES1)
                Comp_ULD_TR |= Comp_ULD_CMP_TRIM1_DIR;
            #endif /* CY_PSOC3_ES2 || CY_PSOC5_ES1 */
            
            /* PSoC3 ES3 or later, PSoC5 ES2 or later */
            #if (CY_PSOC3_ES3 || CY_PSOC5_ES2)
                Comp_ULD_TR0 |= Comp_ULD_CMP_TR0_DIR;
            #endif /* CY_PSOC3_ES3 || CY_PSOC5_ES2 */
        }
    }
    else
    {
        /* if comparator output is low, positive offset adjust is required */
        if ( cmpState == 0u )
        {
            /* PSoC3 ES2 or early, PSoC5 ES1 or early */
            #if (CY_PSOC3_ES2 || CY_PSOC5_ES1)
                Comp_ULD_TR |= Comp_ULD_CMP_TRIM2_DIR; 
            #endif /* CY_PSOC3_ES2 || CY_PSOC5_ES1 */
            
            /* PSoC3 ES3 or later, PSoC5 ES2 or later */
            #if (CY_PSOC3_ES3 || CY_PSOC5_ES2)
                Comp_ULD_TR1 |= Comp_ULD_CMP_TR1_DIR;
            #endif /* CY_PSOC3_ES3 || CY_PSOC5_ES2 */
        }
    }

    /* Increment trim value until compare output changes state */
    for ( trimCnt = 0; trimCnt < 7; trimCnt++ )
    {
        if (nibble == 0u)
        {
            /* PSoC3 ES2 or early, PSoC5 ES1 or early */
            #if (CY_PSOC3_ES2 || CY_PSOC5_ES1)
                Comp_ULD_TR += 1u;
            #endif /* CY_PSOC3_ES2 || CY_PSOC5_ES1 */
            
            /* PSoC3 ES3 or later, PSoC5 ES2 or later */
            #if (CY_PSOC3_ES3 || CY_PSOC5_ES2)
                Comp_ULD_TR0 += 1u;
            #endif /* CY_PSOC3_ES3 || CY_PSOC5_ES2 */
        }
        else
        {
            /* PSoC3 ES2 or early, PSoC5 ES1 or early */
            #if (CY_PSOC3_ES2 || CY_PSOC5_ES1)
                Comp_ULD_TR += 0x10u;
            #endif /* CY_PSOC3_ES2 || CY_PSOC5_ES1 */
            
            /* PSoC3 ES3 or later, PSoC5 ES2 or later */
            #if (CY_PSOC3_ES3 || CY_PSOC5_ES2)
                Comp_ULD_TR1 += 0x10u;
            #endif /* CY_PSOC3_ES3 || CY_PSOC5_ES2 */
        }
        
        CyDelay(1);
        
        /* Check for change in comparator output */
        if ((Comp_ULD_WRK & Comp_ULD_CMP_OUT_MASK) != cmpState)
        {
            break;      /* output changed state, trim phase is complete */
        }        
    }    
}


/*******************************************************************************
* Function Name: Comp_ULD_ZeroCal
********************************************************************************
*
* Summary:
*  This function calibrates the offset of the Analog Comparator.
*
* Parameters:  
*  None
*
* Return:  
*  (uint16)  value written in trim register when calibration complete.
*
* Theory: 
*  This function is used to optimize the calibration for user specific voltage
*  range.  The comparator trim is adjusted to counter transistor offsets
*   - offset is defined as positive if the output transitions to high before inP
*     is greater than inN
*   - offset is defined as negative if the output transitions to high after inP
*     is greater than inP
*
*  The Analog Comparator provides 1 byte for offset trim.  The byte contains two
*  4 bit trim fields - one is a course trim and the other allows smaller
*  offset adjustments.
*  - low nibble - fine trim for Fast mode, course trim for Slow mode
*  - high nibble - fine trim for Slow mode, course trim
*    for Fast mode
*  - trim[3] selects positive or negative offset adjust
*  - for low nibble: trim[3]is set high to add negative offset
*  - for high nibble: trim[3]is set high to add positive offset  
*
*  Trim algorithm is a two phase process
*  The first phase performs course offset adjustment
*  The second phase serves one of two purposes depending on the outcome of the
*  first phase
*  - if the first trim value was maxed out without a comparator output 
*    transition, more offset will be added by adjusting the second trim value.
*  - if the first trim phase resulted in a comparator output transition, the
*    second trim value will serve as fine trim (in the opposite direction)to
*    ensure the offset is < 1 mV.
*
*  Trim Process:   
*  1) User applies a voltage to the negative input.  Voltage should be in the
*     comparator operating range or an average of the operating voltage range.
*  2) Clear registers associated with analog routing to the positive input.
*  3) Disable Hysteresis
*  4) Set the calibration bit to short the negative and positive inputs to
*     the users calibration voltage.
*  5) Clear the TR register  ( TR = 0x00 )
*  ** SLOW MODE
*  6) Check if compare output is high, if so, set trim[3] of lower 
*     nibble to a 1.
*  7) Increment trim[2:0] of lower nibble until the compare output changes
*     (course trim).
*  8) Check if compare output is low, if so, set trim[3] of higher 
*     nibble to a 1.
*  9) Increment trim[2:0] of higher nibble until the compare output changes 
*     state (fine trim)
*  ** FAST MODE - change order of steps 6,7 vs. steps 8,9
*
* Side Effects:
*  Routine clears analog routing associated with the comparator positive input.  
*  This may affect routing of signals from other components that are connected
*  to the positive input of the comparator.
*
* Reentrant:
*  Yes
*
*******************************************************************************/
uint16 Comp_ULD_ZeroCal(void) 
{
    uint8 tmpSW0;
    uint8 tmpSW2;
    uint8 tmpSW3;
    uint8 tmpCR;

    /* Save a copy of routing registers associated with inP */
    tmpSW0 = Comp_ULD_SW0;
    tmpSW2 = Comp_ULD_SW2;
    tmpSW3 = Comp_ULD_SW3;

     /* Clear routing for inP, retain routing for inN */
    Comp_ULD_SW0 = 0x00u;
    Comp_ULD_SW2 = 0x00u;
    Comp_ULD_SW3 = tmpSW3 & ~Comp_ULD_CMP_SW3_INPCTL_MASK;

    /* Preserve original configuration
     * - turn off Hysteresis
     * - set calibration bit - shorts inN to inP
    */
    tmpCR = Comp_ULD_CR;
    Comp_ULD_CR |= (Comp_ULD_CAL_ON | Comp_ULD_HYST_OFF);
    
    /* Write default low values to trim register - no offset adjust */
    /* PSoC3 ES2 or early, PSoC5 ES1 or early */
    #if (CY_PSOC3_ES2 || CY_PSOC5_ES1)
        Comp_ULD_TR = Comp_ULD_DEFAULT_CMP_TRIM;
    #endif /* CY_PSOC3_ES2 || CY_PSOC5_ES1 */
    
    /* PSoC3 ES3 or later, PSoC5 ES2 or later */
    #if (CY_PSOC3_ES3 || CY_PSOC5_ES2)
        Comp_ULD_TR0 = Comp_ULD_DEFAULT_CMP_TRIM;
        Comp_ULD_TR1 = Comp_ULD_DEFAULT_CMP_TRIM;
    #endif /* CY_PSOC3_ES3 || CY_PSOC5_ES2 */

    /* Two phase trim - mode determines which value is trimmed first */   
    if ( (Comp_ULD_CR & Comp_ULD_PWR_MODE_MASK) == Comp_ULD_PWR_MODE_FAST)
    {
        Comp_ULD_trimAdjust(1);          /* course trim */
        Comp_ULD_trimAdjust(0);          /* fine trim */
    }
    else /* default to trim for slow mode */
    {
        Comp_ULD_trimAdjust(0);          /* course trim */
        Comp_ULD_trimAdjust(1);          /* fine trim */
    }
   
    /* Restore Config Register */
    Comp_ULD_CR = tmpCR;
    
    /* Restore routing registers for inP */
    Comp_ULD_SW0 = tmpSW0;
    Comp_ULD_SW2 = tmpSW2;
    Comp_ULD_SW3 = tmpSW3;
    
    /* PSoC3 ES2 or early, PSoC5 ES1 or early */
    #if (CY_PSOC3_ES2 || CY_PSOC5_ES1)
        return (uint16) Comp_ULD_TR;
    #endif /* CY_PSOC3_ES2 || CY_PSOC5_ES1 */
    
    /* PSoC3 ES3 or later, PSoC5 ES2 or later */
    #if (CY_PSOC3_ES3 || CY_PSOC5_ES2)
        return ((Comp_ULD_TR0)|(Comp_ULD_TR1));        
    #endif /* CY_PSOC3_ES3 || CY_PSOC5_ES2 */
}


/*******************************************************************************
* Function Name: Comp_ULD_LoadTrim
********************************************************************************
*
* Summary:
*  This function stores a value in the Analog Comparator trim register.
*
* Parameters:  
*  uint8    trimVal - trim value.  This value is the same format as the value 
*           returned by the _ZeroCal routine.
*
* Return:  
*  None
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Comp_ULD_LoadTrim(uint16 trimVal) 
{
    /* Stores value in the Analog Comparator trim register */
    /* PSoC3 ES2 or early, PSoC5 ES1 or early */
    #if (CY_PSOC3_ES2 || CY_PSOC5_ES1)
        Comp_ULD_TR = (uint8) trimVal;
    #endif /* CY_PSOC3_ES2 || CY_PSOC5_ES1 */
    
    /* PSoC3 ES3 or later, PSoC5 ES2 or later */
    #if (CY_PSOC3_ES3 || CY_PSOC5_ES2)
        /* Stores value in the Analog Comparator trim register for P-type load */
        Comp_ULD_TR0 = (uint8) trimVal;
        
        /* Stores value in the Analog Comparator trim register for N-type load */
        Comp_ULD_TR1 = (uint8) (trimVal >> 8); 
    #endif /* CY_PSOC3_ES3 || CY_PSOC5_ES2 */
}


#if (CY_PSOC3)
/*******************************************************************************
* Function Name: Comp_ULD_PwrDwnOverrideEnable
********************************************************************************
*
* Summary:
*  This is the power down over-ride feature. This function ignores sleep 
*  parameter and allows the component to stay active during sleep mode.
*
* Parameters:  
*  None
*
* Return:  
*  None
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Comp_ULD_PwrDwnOverrideEnable(void) 
{
    /* Set the pd_override bit in CMP_CR register */
    Comp_ULD_CR |= Comp_ULD_PWRDWN_OVRD;
}


/*******************************************************************************
* Function Name: Comp_ULD_PwrDwnOverrideDisable
********************************************************************************
*
* Summary:
*  This is the power down over-ride feature. This allows the component to stay
*  inactive during sleep.
*
* Parameters:  
*  None
*
* Return:  
*  None
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void Comp_ULD_PwrDwnOverrideDisable(void) 
{
    /* Reset the pd_override bit in CMP_CR register */
    Comp_ULD_CR &= ~Comp_ULD_PWRDWN_OVRD;
}
#endif /* (CY_PSOC3) */


/* [] END OF FILE */
