/*******************************************************************************
* File Name: PGA_follower.h  
* Version 1.80
*
* Description:
*  This file contains the function prototypes and constants used in
*  the OpAmp (Analog Buffer) Component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_OPAMP_PGA_follower_H) 
#define CY_OPAMP_PGA_follower_H 

#include "cytypes.h"
#include "cyfitter.h"

/* Check to see if required defines such as CY_PSOC5LP are available */
/* They are defined starting with cy_boot v3.0 */
#if !defined (CY_PSOC5LP)
    #error Component OpAmp_v1_80 requires cy_boot v3.0 or later
#endif /* (CY_ PSOC5LP) */


/***************************************
*   Data Struct Definition
***************************************/

/* Low power Mode API Support */
typedef struct _PGA_follower_backupStruct
{
    uint8   enableState;
}   PGA_follower_BACKUP_STRUCT;


/**************************************
*        Function Prototypes 
**************************************/

void PGA_follower_Start(void)           ;
void PGA_follower_Stop(void)            ;
void PGA_follower_SetPower(uint8 power) ;
void PGA_follower_Sleep(void)           ;
void PGA_follower_Wakeup(void)          ;
void PGA_follower_SaveConfig(void)      ;
void PGA_follower_RestoreConfig(void)   ;
void PGA_follower_Init(void)            ;
void PGA_follower_Enable(void)          ;


/**************************************
*           API Constants
**************************************/

/* Power constants for SetPower() function */
#if (CY_PSOC3 || CY_PSOC5LP)
    #define PGA_follower_LPOCPOWER (0x00u)
    #define PGA_follower_LOWPOWER  (0x01u)
    #define PGA_follower_MEDPOWER  (0x02u)
    #define PGA_follower_HIGHPOWER (0x03u)
#else
    #define PGA_follower_LOWPOWER  (0x01u)
    #define PGA_follower_MEDPOWER  (0x02u)
    #define PGA_follower_HIGHPOWER (0x00u)
#endif /* (CY_PSOC3 || CY_PSOC5LP) */


/**************************************
*           Parameter Defaults        
**************************************/
#if (CY_PSOC3 || CY_PSOC5LP)
    #define PGA_follower_DEFAULT_POWER    (1u)
#endif  /* (CY_PSOC3 || CY_PSOC5LP) */

#if (CY_PSOC5A)
    #if (1u == 3)
        #define PGA_follower_DEFAULT_POWER (0u)
    #endif    /* (1u == 3) */
#endif   /* (CY_PSOC5A) */

#define PGA_follower_DEFAULT_MODE     (1u)


/**************************************
*             Registers
**************************************/

#define PGA_follower_CR_REG        (* (reg8 *) PGA_follower_ABuf__CR)
#define PGA_follower_CR_PTR        (  (reg8 *) PGA_follower_ABuf__CR)

#define PGA_follower_MX_REG        (* (reg8 *) PGA_follower_ABuf__MX)
#define PGA_follower_MX_PTR        (  (reg8 *) PGA_follower_ABuf__MX)

#define PGA_follower_SW_REG        (* (reg8 *) PGA_follower_ABuf__SW)
#define PGA_follower_SW_PTR        (  (reg8 *) PGA_follower_ABuf__SW)

/* Active mode power manager register */
#define PGA_follower_PM_ACT_CFG_REG    (* (reg8 *) PGA_follower_ABuf__PM_ACT_CFG)
#define PGA_follower_PM_ACT_CFG_PTR    (  (reg8 *) PGA_follower_ABuf__PM_ACT_CFG)

/* Alternative mode power manager register */
#define PGA_follower_PM_STBY_CFG_REG    (* (reg8 *) PGA_follower_ABuf__PM_STBY_CFG)
#define PGA_follower_PM_STBY_CFG_PTR    (  (reg8 *) PGA_follower_ABuf__PM_STBY_CFG)

/* ANIF.PUMP.CR1 Pump Configuration Register 1 */
#define PGA_follower_PUMP_CR1_REG  (* (reg8 *) CYDEV_ANAIF_CFG_PUMP_CR1)
#define PGA_follower_PUMP_CR1_PTR  (  (reg8 *) CYDEV_ANAIF_CFG_PUMP_CR1)

/* Trim register defines */
#define PGA_follower_TR0_REG       (* (reg8 *) PGA_follower_ABuf__TR0)
#define PGA_follower_TR0_PTR       (  (reg8 *) PGA_follower_ABuf__TR0)

#define PGA_follower_TR1_REG       (* (reg8 *) PGA_follower_ABuf__TR1)
#define PGA_follower_TR1_PTR       (  (reg8 *) PGA_follower_ABuf__TR1)

/* PM_ACT_CFG (Active Power Mode CFG Register) mask */ 
#define PGA_follower_ACT_PWR_EN   PGA_follower_ABuf__PM_ACT_MSK 

/* PM_STBY_CFG (Alternative Active Power Mode CFG Register) mask */ 
#define PGA_follower_STBY_PWR_EN   PGA_follower_ABuf__PM_STBY_MSK 


/**************************************
*       Register Constants
**************************************/

/* CX Analog Buffer Input Selection Register */

/* Power mode defines */
#define PGA_follower_PWR_MASK           (0x03u)
#if (CY_PSOC3 || CY_PSOC5LP)
    #define PGA_follower_PWR_LPOC        (0x00u)
    #define PGA_follower_PWR_LOW         (0x01u)
    #define PGA_follower_PWR_MEDIUM      (0x02u)
    #define PGA_follower_PWR_HIGH        (0x03u)
#else
    #define PGA_follower_PWR_LOW         (0x01u)
    #define PGA_follower_PWR_MEDIUM      (0x02u)
    #define PGA_follower_PWR_HIGH        (0x00u)
#endif   /* (CY_PSOC3 || CY_PSOC5LP) */

/* MX Analog Buffer Input Selection Register */

/* Bit Field  MX_VN */
#define PGA_follower_MX_VN_MASK       (0x30u)
#define PGA_follower_MX_VN_NC         (0x00u)
#define PGA_follower_MX_VN_AG4        (0x10u)
#define PGA_follower_MX_VN_AG6        (0x20u)

/* Bit Field  MX_VP */
#define PGA_follower_MX_VP_MASK       (0x0Fu)
#define PGA_follower_MX_VP_NC         (0x00u)
#define PGA_follower_MX_VP_VREF       (0x09u)
#define PGA_follower_MX_VP_AG4        (0x01u)
#define PGA_follower_MX_VP_AG5        (0x02u)
#define PGA_follower_MX_VP_AG6        (0x03u)
#define PGA_follower_MX_VP_AG7        (0x04u)
#define PGA_follower_MX_VP_ABUS0      (0x05u)
#define PGA_follower_MX_VP_ABUS1      (0x06u)
#define PGA_follower_MX_VP_ABUS2      (0x07u)
#define PGA_follower_MX_VP_ABUS3      (0x08u)

/* SW Analog Buffer Routing Switch Reg */

/* Bit Field  SW */
#define PGA_follower_SW_MASK          (0x07u) 
#define PGA_follower_SW_SWINP         (0x04u)  /* Enable positive input */
#define PGA_follower_SW_SWINN         (0x02u)  /* Enable negative input */

/* Pump configuration register masks */
#define PGA_follower_PUMP_CR1_CLKSEL  (0x40u)
#define PGA_follower_PUMP_CR1_FORCE   (0x20u)
#define PGA_follower_PUMP_CR1_AUTO    (0x10u)

#endif /* CY_OPAMP_PGA_follower_H */


/* [] END OF FILE */
