/* ========================================
 *
 * Copyright RMS&T/RSSD, BARC,2012
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF Bhabha Atomic Research Centre.
 *
 * ========================================
*/

#include <device.h>
#include "stdio.h"


/*--includes for sheet 1 Counters--*/
#include "Counter_SCA_1_S1.h"
#include "Counter_SCA_2_S1.h"

/*--includes for sheet 2 Counters--*/
#include "Counter_SCA_1_S2.h"
#include "Counter_SCA_2_S2.h"

#define BUFFER_LEN 12 // NUMBER OF BYTES THAT CAN BE SENT AND RECEIVED 
#define STORE_TIME 15 // 15 minutes data to be dumped in memory


/*--S1--*/

int16 count_val_1_S1;
int16 count_val_2_S1;


int16 count_val_S1; // for counts in window of SCA 1


/*--S2--*/

uint8 halt;
int16 count_val_1_S2;
int16 count_val_2_S2;


int16 count_val_S2; // for counts in window of SCA 2

/*--variables for USBUART--*/
//uint8 *ptrUdata[30];
char countstr1[30],countstr2[30];
//char outstr[30]; //pointer to send out of window counts data
/*--to get threshold levels through USB--*/

char cmd;
static uint16 del_mem= 15;
uint8 txdat(uint16); // PROTOTYPE DECLARATION FOR FUNCTION TO TRANSFER 
					 // DATA TO VIRTUAL COMMPORT 

int main()
{
	  /* Hardware initialization */
		
		/*---Sheet1-------------------*/
		VDAC8_LLD_S1_Start();
		VDAC8_ULD_S1_Start();
		
		Comp_LLD_S1_Start();
		Comp_ULD_S1_Start();		
		
					
		Counter_SCA_1_S1_Start();
		Counter_SCA_2_S1_Start();
				
		/*-------- Display: LCD and USBUART ---------*/		
		
		LCD_Char_SCA_S1_Start();
		
		USBUART_Start(0, USBUART_5V_OPERATION); 
		
		/*----instantiaite EEPROM-----*/
		EEPROM_Start();
		
		/*---Sheet2-------------------*/
		
		VDAC8_LLD_S2_Start();
		VDAC8_ULD_S2_Start();
		
		
		Comp_LLD_S2_Start();
		Comp_ULD_S2_Start();		
	
		
		Counter_SCA_1_S2_Start();
		Counter_SCA_2_S2_Start();
				
					
		/* Enable the global interrupt */
		CYGlobalIntEnable;
	
				
	
		
		/*--wait while USB configuration not received--*/
		while(!USBUART_GetConfiguration());
		/*---------------------------------------------*/
		
		USBUART_CDC_Init(); // initialize CDC interface
		
		cmd= 'M';	
		
		for(;;)
		
		{		
		
			halt= 0x00; //flag to stop autoreport
			
		/*--------------------qualify control command--------------------------*/
					/*===setting thresholds for comparison===*/	
							switch (cmd)
							
							{
							
							/*------Window of interest SCA 1 and SCA 2------*/
									
							 case 'd':
							  		
							/*-------default window SCA 1 and SCA 2-----------*/		
									USBUART_PutString("\r\nDefault\r\nC1\tC2");// title the counts from channel 1 and 2
								
									
									/*SCA-1------*/
									VDAC8_LLD_S1_SetValue(0x3F); // set window from 1V(LLD) to 
									VDAC8_ULD_S1_SetValue(0x5E); // 1.5V(ULD) fixed for SCA-1
									
									
									/*SCA-2-----*/
									VDAC8_LLD_S2_SetValue(0x7D); // set window from 2V(LLD) to 
									VDAC8_ULD_S2_SetValue(0x9C); // 2.5V(ULD) fixed for SCA-2
									
									break;
							 
							 case 'l':
							  
							  /*-----Low energy window SCA 1 and SCA 2--------*/

									USBUART_PutString("\r\nLow energy\r\nC1\t\tC2");// title the counts from channel 1 and 2
									
									/*SCA-1------*/
									VDAC8_LLD_S1_SetValue(0x03); // set window from 50mV(LLD) to 
							 		VDAC8_ULD_S1_SetValue(0x0F); // 240mv(ULD) fixed for SCA-1
										
									
									/*SCA-2-----*/
									VDAC8_LLD_S2_SetValue(0x10); // set window from 250Mv(LLD) to 
									VDAC8_ULD_S2_SetValue(0x1F); // 500mv(ULD) fixed for SCA-2
									
									break;
									
							 case 'h':
							  
							  /*-----High energy window SCA 1 and SCA 2--------*/
								
									USBUART_PutString("\r\nHigh Energy\r\nC1\t\tC2");// title the counts from channel 1 and 2
									
									/*SCA-1------*/
									VDAC8_LLD_S1_SetValue(0xDB); // set window from 3500mV(LLD) to 
									VDAC8_ULD_S1_SetValue(0xFF); // 4000mv(ULD) fixed for SCA-1
									
									
									/*SCA-2-----*/
									VDAC8_LLD_S2_SetValue(0x9C); // set window from 4000mV(LLD) to 
									VDAC8_ULD_S2_SetValue(0xBB); // 4080mv(ULD) fixed for SCA-1
									
									break;									
							
									
									
						   case 'M' : default:  //  command menu for user to select  //
							  									
									USBUART_PutString
										(
										  " \r\n Enter d for Default I:1-1.5V,II:2-2.5V"
								  		  "	\r\n Enter l for Low Energy I:50-250 mV,II:250-500mV"
								  		  " \r\n Enter h for High Energy I:3.5-4V,II:4-4.08V "
								  		  " \r\n any key/M for menu "
										  
										 );							
							
										halt= 0x01;
									break;
							}
							
		/*-------------------counting and reporting---------------------------------*/
				/*===setting time for counting ,countingand reporting===*/
		
		
		do{
			    /*---------------For channel 1&2------------------------*/
					Counter_SCA_1_S1_Init();
					Counter_SCA_2_S1_Init();
					Counter_SCA_1_S2_Init();
					Counter_SCA_2_S2_Init();
					
					Counter_SCA_1_S1_WriteCounter (0x0000);
					Counter_SCA_2_S1_WriteCounter (0x0000);
					Counter_SCA_1_S2_WriteCounter (0x0000);
					Counter_SCA_2_S2_WriteCounter (0x0000);
					
					
					CyDelay(10000);
//					
//						if(Comp_LLD_S1_GetCompare()||Comp_LLD_S2_GetCompare())
//						{
							count_val_2_S1= Counter_SCA_2_S1_ReadCounter();
							count_val_1_S1= Counter_SCA_1_S1_ReadCounter();
							count_val_2_S2= Counter_SCA_2_S2_ReadCounter();
							count_val_1_S2= Counter_SCA_1_S2_ReadCounter();
							
							if(count_val_2_S2>=count_val_1_S2 &&count_val_2_S1>=count_val_1_S1)
							{
								count_val_S1= count_val_2_S1-count_val_1_S1;						
								count_val_S2= count_val_2_S2-count_val_1_S2;
							}
//						}
					
					/*------------------report on USB------------------*/
					if (USBUART_CDCIsReady() != 0u&& halt!=1u)   // send this data to USB port
						
			 			{
					
							sprintf(countstr1,"\r\n %u\t%u",count_val_S1,count_val_S2); // convert count value to string
							USBUART_PutString(countstr1);
					
					/*--------------duplicate on LCD------------------------*/
						
						//	
							LCD_Char_SCA_S1_ClearDisplay();
						//
						LCD_Char_SCA_S1_Position(1, 0);
						LCD_Char_SCA_S1_PrintNumber(count_val_S1);
						LCD_Char_SCA_S1_Position(1, 5);
						LCD_Char_SCA_S1_PrintNumber(count_val_S2);
						}
						
//		CyDelay(100);
		}while(!USBUART_DataIsReady());
				
				LCD_Char_SCA_S1_Position(0,0); //update for next
				LCD_Char_SCA_S1_PrintString("Mode ");
				LCD_Char_SCA_S1_PutChar(cmd);
				
		/*-------------------------------------------------------------*/
			
		/*--------------------get control commands---------------------*/
					
		cmd = USBUART_GetChar();

		}
	
}

/*---use later EEPROM--*/
int mem_delay(uint8 y,k)
{
	y=15;
	for (k=y;k--;k=0)
	{
		CyDelay(60000);
	}
	return(0);
}

/* [] END OF FILE */
